package org.steveshipway.blueprints.directive;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;


// Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
//import org.steveshipway.blueprints.util18.*;
//import net.minecraft.block.state.IBlockState;
//import net.minecraft.util.BlockPos;

public class MobDirective  extends Directive {
	private String mobdata;
	private NBTTagCompound nbt = null;
	private BlockPos offset;
	private boolean haserror = false;
	
	static Pattern nbtpattern = Pattern.compile("(?i)^\\s*(?:mob|entity)?\\s*(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s*(\\S+\\.nbt|\\{.*\\})\\s*$");
	
	public MobDirective(String definition, String bppath,Logger logger) {
		// parse options
		// MOB x y z filename.nbt
		// MOB x y z { nbt }
		// MOB x y z name
		Matcher m;
		
		this.mobdata = definition;
		
		// parse explicit NBT for the spawner
		m = nbtpattern.matcher(definition);
		if ( m.find() ) {
			this.mobdata = m.group(4);
			try {
				this.offset = new BlockPos(Integer.parseInt(m.group(1)),Integer.parseInt(m.group(2)),Integer.parseInt(m.group(3)));
			} catch ( Exception e ) {
				logger.error("Invalid offset format in MOB definition '"+definition+"'");
				this.haserror = true;
				return;
			}
			this.nbt = Util.parseNBT(m.group(4),bppath,logger);
			if( this.nbt == null ) {
				logger.error("Supplied NBT in "+m.group(4)+" does not parse");
				this.haserror = true;			
				return;
			}
			if( this.nbt.getString("id") == null || this.nbt.getString("id").length()<1 ) {
				logger.error("Supplied NBT in "+m.group(4)+" does not specify an id");
				this.haserror = true;
			}
			return;
		} else {
			logger.error("Invalid format in MOB definition '"+definition+"'");
			this.haserror = true;			
		}
	}
	public MobDirective(BlockPos offset, NBTTagCompound nbt) {
		this.offset = offset;
		this.nbt = nbt;
		this.mobdata = nbt.toString();
		//CheshireCat.logger.info("Creating new mob directive: "+offset.getX()+","+offset.getY()+","+offset.getZ()+" : "+nbt.toString());
	}
	
	@Override
	public boolean build(World world, BlockPos location, int r, Logger logger) {
		IBlockState blockState = null;
		BlockPos blockPos = new BlockPos(location); 
		boolean rv;
		NBTTagCompound nbtTag = null;
		Entity entity;
		
		if( world.isRemote ) {
    		logger.error("Mob placement called with remote world?  Should never happen.");
			return false;
		}
		if( this.haserror || (this.nbt==null) ) {
			logger.error("Mob definition has an error, cannot spawn");
			return false;
		}
		
		// get block position, rotated
		blockPos = rotateAndAdd(this.offset,r,location);

		// set the position tags
		nbtTag = (NBTTagCompound) this.nbt.copy();
		NBTTagList posList = new NBTTagList();
		posList.appendTag(new NBTTagDouble((double)blockPos.getX()+0.5D));
		posList.appendTag(new NBTTagDouble((double)blockPos.getY()));
		posList.appendTag(new NBTTagDouble((double)blockPos.getZ()+0.5D));
		nbtTag.setTag("Pos",posList);
		// is this sufficient?  Can we rely on the NBT being correct?
		
		// create the entity
		entity = EntityList.createEntityFromNBT(nbtTag, world);
		if( entity == null ) {
			logger.error("Unable to create Entity from the NBT: '"+this.nbt.toString()+"'");
			//return false;
		} else {
			entity.setPosition((double) blockPos.getX()+0.5D,(double) blockPos.getY(),(double) blockPos.getZ()+0.5D);		
			world.spawnEntityInWorld(entity);
			//logger.info("Spawning in the world: "+this.nbt.getString("id"));
		}
		
		return true;
	}
	
	@Override
	public String asText() {
		String s = "MOB ";
		if( this.haserror || (this.nbt == null)) {
			return "# MOB undefined ";
		}
		s += " " + offset.getX() + "," + offset.getY() + "," + offset.getZ();
		if( this.mobdata != null ) {
			s += " " + this.mobdata;
		}
		return s;
	}
}
